// SNF4CGP/main.cpp
//
// (C) Copyright 2009 ARM Research Labs, LLC
// See www.armresearch.com for more information.
//
// SNF4CGP is a "filter helper" for Stalker's CommuniGate Pro
// It implements a full SNFMulti antispam/antimalware engine and multi-threaded
// scanner. SNFMulti scan results are mapped to appropriate responses.

#include <iostream>
#include <string>
#include "ExecutiveProcess.hpp"

using namespace std;                                                            // Introduce standard namespace.

const string SNF4CGP_VERSION_INFO = "CGPSNF Version 1.4.0 Build: " __DATE__ " " __TIME__;

const int ConfigPathArgNumber = 1;
const int MaxArgcNumber = 2;
const string DefaultConfigFile = "snf_engine.xml";

bool CommandLineIsValid(int argc, char*argv[]) {                                // Validate the command line.
    if(argc > MaxArgcNumber) {
        cout << "* CGPSNF - Bad command line.  Use:" << endl
	     << "\tCGPSNF <path_to_config_file>" << endl
	     << "to use the specified config file, or" << endl
	     << "\tCGSNF" << endl
	     << "to use the default config file '" << DefaultConfigFile << "' "
	     << "in the same directory as the executable."
	     << endl;
        return false;
    }
    return true;
}

string ConfigPathFromArgv0(const string Argv0) {                                // Return the config file name in
    const string Slashes = "/\\";                                               // the same directory as the executable.
    string::size_type LastSlashIndex = Argv0.find_last_of(Slashes);
    if (string::npos == LastSlashIndex) {                                       // If there's no slash in the path...
	return DefaultConfigFile;
    }
    return Argv0.substr(0, LastSlashIndex + 1) + DefaultConfigFile;
}

string getConfigPath(int argc, char *argv[]) {                                  // Returns the config file name.
    bool ConfigurationPathProvided = (ConfigPathArgNumber < argc);
    if (ConfigurationPathProvided) {
	return string(argv[ConfigPathArgNumber]);
    }
    return ConfigPathFromArgv0(argv[0]);
}

void go(string ConfigurationPath) {                                             // Application's main thread.
    try {
        ExecutiveProcess SNF4CGP(SNF4CGP_VERSION_INFO, ConfigurationPath);
        SNF4CGP.doStartup();
        SNF4CGP.doProcessing();
        SNF4CGP.doShutdown();
    }
    catch(exception& e) {                                                       // Display standard exceptions.
        cout << "* CGPSNF Exception: " << e.what() << endl;
    }
    catch(...) {                                                                // Display unusual exceptions.
        cout << "* CGPSNF Unusual Excetption!" << endl;
    }
}

int main(int argc, char* argv[]) {                                              // Main entry point.
    if(CommandLineIsValid(argc, argv)) go(getConfigPath(argc, argv));           // With a good command line we go.
    return 0;                                                                   // Then we're done.
}
